/*
 * Main.fx
 *
 * @author Dmitry Danileyko
 *
 * Created on 26.11.2009, 21:58:21
 */

package com.pcworld.chineseplayer;

import javafx.stage.Stage;
import javafx.scene.Scene;
import javafx.stage.StageStyle;
import javafx.scene.paint.Color;
import javafx.scene.shape.Rectangle;
import javafx.scene.Group;
import javafx.scene.shape.Circle;
import javafx.scene.shape.Arc;
import javafx.scene.shape.ArcType;
import javafx.scene.effect.Lighting;
import javafx.scene.effect.light.DistantLight;
import javafx.scene.shape.Polyline;
import javafx.scene.effect.DropShadow;
import javafx.scene.shape.Line;
import javafx.scene.input.MouseButton;
import javafx.scene.input.MouseEvent;
import javafx.scene.control.ListView;
import javax.swing.JFileChooser;
import javafx.scene.media.Media;
import javafx.scene.media.MediaPlayer;
import javafx.scene.media.MediaView;
import javafx.scene.input.KeyEvent;
import javafx.scene.input.KeyCode;
import javafx.scene.control.Slider;
import javafx.scene.text.Text;
import javafx.scene.text.Font;
import javafx.scene.text.TextAlignment;
import javafx.animation.Timeline;
import javafx.animation.KeyFrame;
import javafx.scene.effect.Bloom;
import javax.swing.filechooser.FileFilter;
import java.io.File;

/**
 * @author Dmitry Danileyko
 */
var fileList : String[];
var playPause : Boolean = true;
var beginPointX : Number;
var beginPointY : Number;
var mediaDurStep : Double;
var track : String;

var player : Group = new Group();
var display : Rectangle = new Rectangle();
var case : Rectangle = new Rectangle();
var buttonPlay : Circle = new Circle();
var buttonPrev : Arc = new Arc();
var buttonNext : Arc = new Arc();
var polylinePlay : Polyline = new Polyline();
var groupPause : Group = new Group();
var groupNext : Group = new Group();
var groupPrev : Group = new Group();
var groupX : Group = new Group();
var group_ : Group = new Group();
var groupDots : Group = new Group();
var listviewPlaylist : ListView = new ListView();
var mediaplayer : MediaView = new MediaView();
var sliderVolume : Slider = new Slider();
var trackName : Text = new Text();
var lineProgress : Rectangle = new Rectangle();
var rectProgress : Rectangle = new Rectangle();

var timelineProgress = Timeline {
	repeatCount: Timeline.INDEFINITE
	keyFrames : [
		KeyFrame {
			time : bind Duration.valueOf(mediaDurStep);
			canSkip : true
			action: function () {
                if (rectProgress.layoutX >= lineProgress.layoutX - rectProgress.width / 2 + 350) then {
                } else {
                    rectProgress.layoutX++
                }
            }
		}
	]
}

function resumeMedia(position : Number){
    mediaplayer.mediaPlayer.currentTime = Duration.valueOf(position * mediaDurStep);
    timelineProgress.play();
}

function playMedia(mediaSrc : String){
    rectProgress.layoutX = lineProgress.layoutX - rectProgress.width / 2;
    playPause = true;
    mediaplayer.mediaPlayer.stop();
    mediaplayer.mediaPlayer.media = Media{source: "file:/{mediaSrc.replace("\\","/").replace(" ","%20")}"};
    mediaDurStep = mediaplayer.mediaPlayer.media.duration.toMillis() / 350;
    mediaplayer.mediaPlayer.play();
    timelineProgress.playFromStart();
    track = mediaSrc.substring(mediaSrc.lastIndexOf("\\") + 1);
}

var stage : Stage = Stage {
    title: "Китаский плеир"
    width: 600
    height: 500
    style: StageStyle.TRANSPARENT

    scene: Scene {
        fill: Color.TRANSPARENT
        content: [
            player = Group {
                content: [
                    case = Rectangle {
                        x: 50, y: 50
                        width: 450, height: 150
                        fill: Color.rgb(5, 5, 5)
                        arcHeight:150
                        arcWidth:80
                        stroke:Color.DARKRED
                        strokeWidth:15
                        effect: Lighting {
                            diffuseConstant:1.5
                            light: DistantLight {
                                azimuth: -135
                            }
                            surfaceScale: 4
                            specularConstant:0.1
                        }
                        onMouseEntered: function(evt: MouseEvent): Void {
                            listviewPlaylist.visible = true
                        }
                        onMouseExited: function(evt: MouseEvent): Void {
                            listviewPlaylist.visible = false
                        }

                        onMousePressed: function(e: MouseEvent):Void {
                            beginPointX = e.screenX - stage.x;
                            beginPointY = e.screenY - stage.y;
                        }
                        onMouseDragged: function(e: MouseEvent):Void {
                            stage.x = e.screenX - beginPointX;
                            stage.y = e.screenY - beginPointY;
                        }

                    },
                    display = Rectangle {
                        width: 170, height: 100
                        fill: Color.GREY
                        stroke: Color.GREY
                        layoutX: bind (case.width - display.layoutBounds.width) / 2 + case.y
                        layoutY: bind (case.height - display.layoutBounds.height) / 2 + case.x
                        effect: Lighting {
                            diffuseConstant:0.5
                            light: DistantLight {
                                azimuth: 135
                            }
                            surfaceScale: 1
                            specularConstant:0.1
                        }
                    },
                    trackName = Text {
                        content: bind track;
                        wrappingWidth: 130
                        layoutX: bind display.layoutX + 30
                        layoutY: bind (display.height - trackName.layoutBounds.height) / 2 + display.layoutY + trackName.font.size / 2
                        fill: Color.AQUA
                        textAlignment: TextAlignment.CENTER
                        font: Font{
                            size:14
                        }
                        effect: Bloom{
                        }
                    },
                    sliderVolume = Slider {
                        layoutX: bind display.layoutX + 10
                        layoutY: bind display.layoutY + 10
                        min: 0
                        max: 9
                        value: 0
                        vertical: true
                        height: 80
                    },
                    buttonPlay = Circle {
                        radius: 50
                        fill: Color.DARKRED
                        centerY: bind case.height / 2 + case.y
                        centerX: bind display.layoutX - 70
                        effect: Lighting {
                            diffuseConstant:1.5
                            light: DistantLight {
                                azimuth: -135
                            }
                            surfaceScale: 1
                            specularConstant:0.5
                        }
                        onMouseClicked: function(evt: MouseEvent){
                            if ((evt.clickCount == 1) and (evt.button == MouseButton.PRIMARY)) then {
                                if (mediaplayer.mediaPlayer.paused) then {
                                    mediaplayer.mediaPlayer.play();
                                    timelineProgress.play();
                                    playPause = not playPause;
                                }
                                else {
                                    mediaplayer.mediaPlayer.pause();
                                    timelineProgress.pause();
                                    playPause = not playPause;
                                }
                            }
                        }
                    },
                    polylinePlay = Polyline {
                        visible: bind playPause
                        points : [ buttonPlay.centerX - 20,buttonPlay.centerY - 30,
                                   buttonPlay.centerX + 30,buttonPlay.centerY,
                                   buttonPlay.centerX - 20,buttonPlay.centerY + 30,
                                   buttonPlay.centerX - 20,buttonPlay.centerY - 30 ]
                        strokeWidth: 1.0
                        stroke: Color.GREY
                        fill:Color.WHITE
                        effect: DropShadow {
                            offsetY: 4
                        }

                    },
                    groupPause = Group {
                        layoutX: bind buttonPlay.centerX - groupPause.layoutBounds.width / 2
                        layoutY: bind buttonPlay.centerY - groupPause.layoutBounds.height / 2
                        content: [
                            Rectangle {
                                width: 10, height: 50
                                stroke: Color.GREY
                                strokeWidth: 1.0
                                fill:Color.WHITE
                            },
                            Rectangle {
                                x: 20, y: 0
                                width: 10, height: 50
                                stroke: Color.GREY
                                strokeWidth: 1.0
                                fill:Color.WHITE
                            }
                        ]
                        effect: DropShadow {
                            offsetY: 4
                        }
                        visible: bind not playPause
                    },
                    buttonPrev = Arc {
                        centerX: bind display.layoutX + display.width + 70, centerY: bind case.arcHeight / 2 + case.x
                        radiusX: 50, radiusY: 50
                        startAngle: 90
                        length: 180
                        type: ArcType.CHORD
                        fill: Color.GREEN
                        effect: Lighting {
                            diffuseConstant:1.5
                            light: DistantLight {
                                azimuth: -135
                            }
                            surfaceScale: 1
                        }
                        onMouseClicked: function(evt: MouseEvent): Void {
                            if ((evt.clickCount == 1) and (evt.button == MouseButton.PRIMARY)) then {
                                if (listviewPlaylist.selectedIndex != 0) then {
                                    listviewPlaylist.selectPreviousRow()
                                } else {
                                    listviewPlaylist.selectLastRow();
                                }
                                playMedia(fileList[listviewPlaylist.selectedIndex]);
                            }
                        }
                    },
                    groupPrev = Group {
                        layoutX: bind buttonPrev.centerX - groupPrev.layoutBounds.width / 2
                        layoutY: bind buttonPrev.centerY - groupPrev.layoutBounds.height / 2
                        content: [
                            Polyline {
                                stroke: Color.GREY
                                points : [-5,0, -20,25, -5,50, -5,0 ]
                                strokeWidth: 1.0
                                fill:Color.WHITE
                            },
                            Polyline {
                                stroke: Color.GREY
                                points : [15,0, 0,25, 15,50, 15,0 ]
                                strokeWidth: 1.0
                                fill:Color.WHITE
                            },
                        ]
                        effect: DropShadow { 
                            offsetY: 4
                        }
                    },
                    buttonNext = Arc {
                        centerX: bind display.layoutX + display.width + 70, centerY: bind case.arcHeight / 2 + case.x
                        radiusX: 50, radiusY: 50
                        startAngle: 270,
                        length: 180
                        type: ArcType.OPEN
                        fill: Color.BLUE
                        stroke: Color.BLUE
                        effect: Lighting {
                            diffuseConstant:1.5
                            light: DistantLight { 
                                azimuth: -135
                            }
                            surfaceScale: 1
                        }
                        onMouseClicked: function(evt: MouseEvent): Void {
                            if ((evt.clickCount == 1) and (evt.button == MouseButton.PRIMARY)) {
                                if (listviewPlaylist.selectedIndex != sizeof listviewPlaylist.items - 1) then {
                                    listviewPlaylist.selectNextRow()
                                } else {
                                    listviewPlaylist.selectFirstRow();
                                }
                                playMedia(fileList[listviewPlaylist.selectedIndex]);
                            }
                        }
                    },
                    groupNext = Group {
                        layoutX: bind buttonNext.centerX - groupNext.layoutBounds.width / 2
                        layoutY: bind buttonNext.centerY - groupNext.layoutBounds.height / 2
                        content: [
                            Polyline {
                                points : [42,0, 57,25, 42,50, 42,0 ]
                                strokeWidth: 1.0
                                stroke: Color.GREY
                                fill:Color.WHITE
                            },
                            Polyline {
                                points : [22,0, 37,25, 22,50, 22,0 ]
                                strokeWidth: 1.0
                                stroke: Color.GREY
                                fill:Color.WHITE
                            },
                        ]
                        effect: DropShadow { 
                            offsetY: 4
                        }
                    },
                    groupX = Group {                                                
                        layoutX: bind case.layoutBounds.width - 39
                        layoutY: bind case.layoutBounds.minY - 26
                        content: [
                            Rectangle {
                                arcHeight:8
                                arcWidth:8
                                x: 10, y: 10
                                width: 20, height: 20
                                stroke:Color.GREY
                                fill:Color.DARKRED
                                onMouseClicked: function(evt: MouseEvent): Void {
                                    if ((evt.clickCount == 1) and (evt.button == MouseButton.PRIMARY)) then {
                                        mediaplayer.mediaPlayer.stop();
                                        FX.exit();
                                    }
                                }
                            },
                            Line {
                                startX: 13, startY: 13
                                endX: 27, endY: 27
                                strokeWidth: 1
                                stroke: Color.WHITE
                            },
                            Line {
                                startX: 13, startY: 27
                                endX: 27, endY: 13
                                strokeWidth: 1
                                stroke: Color.WHITE
                            }

                        ]
                    },
                    group_ = Group {                                                
                        layoutX: bind case.layoutBounds.width - 60
                        layoutY: bind case.layoutBounds.minY - 26
                        content: [
                            Rectangle {
                                arcHeight:8
                                arcWidth:8
                                x: 10, y: 10
                                width: 20, height: 20
                                stroke:Color.GREY
                                fill: Color.DARKRED
                                onMouseClicked: function(evt: MouseEvent): Void {
                                    if ((evt.clickCount == 1) and (evt.button == MouseButton.PRIMARY)) then {
                                        stage.iconified = true
                                    }
                                }
                            },
                            Line {
                                startX: 14, startY: 23
                                endX: 26, endY: 23
                                strokeWidth: 1
                                stroke: Color.WHITE
                            }
                        ]
                    },
                    groupDots = Group {
                        layoutX: bind case.layoutBounds.width - 81
                        layoutY: bind case.layoutBounds.minY - 26
                        content: [
                            Rectangle {
                                arcHeight:8
                                arcWidth:8
                                x: 10, y: 10
                                width: 20, height: 20
                                stroke: Color.GREY
                                fill: Color.DARKRED
                                onMouseClicked: function(evt: MouseEvent): Void {
                                    if ((evt.clickCount == 1) and (evt.button == MouseButton.PRIMARY)) then {
                                        var dialogOpen : JFileChooser = new JFileChooser();
                                        dialogOpen.setMultiSelectionEnabled(true);
                                        dialogOpen.setFileFilter(new Mp3FileFilter());
                                        dialogOpen.setAcceptAllFileFilterUsed(false);
                                        if (JFileChooser.APPROVE_OPTION == dialogOpen.showOpenDialog(null)) {
                                            for (file in dialogOpen.getSelectedFiles()){
                                                insert file.getAbsolutePath() into fileList;
                                            }
                                        }
                                    }
                                }
                            },
                            Circle {
                                centerX: 14, centerY: 23
                                radius: 1.5
                                fill: Color.WHITE
                            },
                            Circle {
                                centerX: 20, centerY: 23
                                radius: 1.5
                                fill: Color.WHITE
                            },
                            Circle {
                                centerX: 26, centerY: 23
                                radius: 1.5
                                fill: Color.WHITE
                            },

                        ]
                    },
                    listviewPlaylist = ListView {
                        width: 300
                        height: 200
                        layoutY: bind case.layoutBounds.maxY - 3
                        layoutX: bind (case.width - listviewPlaylist.layoutBounds.width) / 2  + case.y
                        onMouseEntered: function(evt: MouseEvent): Void {
                            listviewPlaylist.visible = true
                        }
                        onMouseExited: function(evt: MouseEvent): Void {
                            listviewPlaylist.visible = false
                        }
                        onMouseClicked: function(evt: MouseEvent): Void {
                            if ((evt.clickCount == 2) and (evt.button == MouseButton.PRIMARY)) then {
                                playMedia(fileList[listviewPlaylist.selectedIndex]);
                            }
                        }
                        onKeyPressed: function(evt: KeyEvent): Void{
                            if ((evt.code == KeyCode.VK_DELETE) and (listviewPlaylist.selectedIndex != -1)) then {
                                delete fileList[listviewPlaylist.selectedIndex]
                            }
                        };
                        items: bind fileList
                    },
                    mediaplayer = MediaView {
                        mediaPlayer : MediaPlayer {
                            volume: bind (9 - sliderVolume.value) / 10
                            onEndOfMedia: function(): Void {
                                timelineProgress.stop();
                                if (listviewPlaylist.selectedIndex != sizeof listviewPlaylist.items - 1){
                                    listviewPlaylist.selectNextRow()
                                } else {
                                    listviewPlaylist.selectFirstRow();
                                }
                                playMedia(fileList[listviewPlaylist.selectedIndex]);
                            }
                            media : Media {
                            }
                        }
                    },
                    lineProgress = Rectangle {
                        layoutX: bind (case.width - lineProgress.layoutBounds.width) / 2 + case.x
                        layoutY: bind case.layoutBounds.maxY - 28
                        width: 350, height: 1
                        fill: Color.GREY
                        arcHeight: 6
                        arcWidth: 6
                    },
                    rectProgress = Rectangle {
                        width: 12, height: 12
                        fill: Color.GRAY
                        strokeWidth: 1
                        stroke: Color.GREY
                        layoutX: lineProgress.layoutX - rectProgress.width / 2
                        layoutY: bind lineProgress.layoutY - rectProgress.height / 2
                        blocksMouse: true

                        onMousePressed: function(e: MouseEvent):Void {
                            timelineProgress.pause();
                            beginPointX = e.sceneX - rectProgress.width / 2;
                        }
                        onMouseDragged: function(e: MouseEvent):Void {
                            var minPosition = lineProgress.layoutX - rectProgress.width / 2;
                            var maxPosition = lineProgress.layoutX - rectProgress.width / 2 + 350;
                            var xPosition = e.sceneX - rectProgress.width / 2;
                            if(xPosition <= minPosition) then {
                                xPosition = minPosition
                            } else if (xPosition >= maxPosition) then {
                                xPosition = maxPosition
                            }
                            rectProgress.layoutX = xPosition;
                        }
                        onMouseReleased: function(e: MouseEvent):Void {
                            resumeMedia(e.sceneX - rectProgress.width / 2 - lineProgress.layoutX - rectProgress.width / 2);
                        }
                    }
                ]
            }
        ]
    }
}

class Mp3FileFilter extends FileFilter{
    override public function accept(file : File) : Boolean{
        if (file.getName().endsWith(".mp3")) then {
			return true;
		};
        if (file.isDirectory()) then {
			return true;
		};
        return false;
    }
    override public function getDescription() : String{
        return "mp3 files";
    }
}